//this module determines whether or not to color in a pixel
//because the pixel is part of a letter
module screenWriter(	input wire		 	iRESET,
						input wire			iSetL,
						input wire		 	iCLK50,
						input wire [8:0] 	iX,
						input wire [7:0] 	iY,
						input wire [17:0] 	iSW,
						input wire			next_key,
						input wire [8:0]	char,
						output wire [17:0]	oLEDR,
						output wire		 	letter	);

// NUM_LETTER_SIZE = 5 because we're using between 2^4 and 2^5 different 
// characters in the cgen lookup table
parameter LETTER_WIDTH = 8, LETTER_HEIGHT = 8, NUM_LETTERS_SIZE = 5;

// width = 640/8 = 80 (each charater is 8 pixels wide)
// height = 480/8 = 60 (each character is 8 pixels high)
//parameter 	SCREEN_WIDTH = 80, SCREEN_HEIGHT = 60;
//parameter	SCREEN_WIDTH_SIZE = 7, SCREEN_HEIGHT_SIZE = 6;
//parameter 	SCREEN_WIDTH = 4, SCREEN_HEIGHT = 4;
//parameter	SCREEN_WIDTH_SIZE = 2, SCREEN_HEIGHT_SIZE = 2;
parameter 	SCREEN_WIDTH = 16, SCREEN_HEIGHT = 1;
parameter	SCREEN_WIDTH_SIZE = 4, SCREEN_HEIGHT_SIZE = 1;

//wire [LETTER_WIDTH-1:0] letters[LETTER_HEIGHT-1:0][NUM_LETTERS-1:0];
wire [LETTER_WIDTH-1:0] letters;


assign oLEDR[17:9] = sentences[0];
assign oLEDR[SCREEN_WIDTH_SIZE-1:0] = xCur;


// address is 12 bits
// zero pad sentences so that it's 8 bits, and then pad address so that it's 12 bits
cgen	characterGenerator(		.address(	{4'b0,(iY&8'h7)+
											{sentences[sX]/*[sY[SCREEN_HEIGHT_SIZE-1:0]]*/,3'b0}}),
								.clock(iCLK50),
								.data(0),
								.wren(0),
								.q(letters)	);

// 0 = no letter
// 1 = A
// 2 = B
// etc..
reg [NUM_LETTERS_SIZE-1:0] sentences[SCREEN_WIDTH-1:0]/*[SCREEN_HEIGHT-1:0]*/;

// position of text cursor
reg [SCREEN_WIDTH_SIZE-1:0] xCur;
//reg [SCREEN_HEIGHT_SIZE-1:0] yCur;

// for resetting sentences
reg [SCREEN_WIDTH_SIZE-1:0] rX;
//reg [SCREEN_HEIGHT_SIZE-1:0] rY;
reg [NUM_LETTERS_SIZE-1:0] value;

reg last_next_key;

always @(posedge next_key)
begin
	xCur <= xCur + 1;
	sentences[xCur] <= char;
end

/*always @ (posedge iCLK50)
begin
	if (iRESET)
	begin
		if (iSW[17]) sentences[rX] <= 9'h1;
		else sentences[rX] <= 9'h0;
		rX <= rX + 1;
		if (rX == SCREEN_WIDTH-1)
		begin
			rX <= 0;
		end
		xCur <= 0;
	end
	else if ((last_next_key != next_key) && next_key)
	begin
		// write to sentences
		sentences[xCur] <= char;
		if (xCur < SCREEN_WIDTH-1) xCur <= xCur + 1;
		else
		begin
			xCur <= 0;
		end
	end
	
	last_next_key <= next_key;
end*/

// address of letter with sentences
wire [SCREEN_WIDTH_SIZE-1:0] sX = iX[SCREEN_WIDTH_SIZE+2:3];
//wire [SCREEN_HEIGHT_SIZE-1:0] sY = iY[SCREEN_HEIGHT_SIZE+2:3];

// address of letter with sentences
//wire [8:0] sX = iX>>3;
//wire [7:0] sY = iY>>3;

//assign letter = ((sentences[sX][sY]>0) & (sentences[sX][sY]<27)) ? letters[SCREEN_WIDTH-1-iX] : 0;
assign letter = (	(iX[8:3] > 10) &
					(iX[8:3] < 27) &
					(iY[7:3] == 15)  &
					(sentences[sX]/*[sY]*/ > 0) &
					(sentences[sX]/*[sY]*/<27)	) ?
					letters[LETTER_WIDTH-1-iX[2:0]] : 0;

endmodule
